@@ -4,8 +4,7 @@
 //!
 //! ## Overview
 //!
-//! The `parse` module provides the [`decompose`] function for parsing markdown documents
-//! and the [`ParsedDocument`] type for accessing parsed content.
+//! The `parse` module provides the [`ParsedDocument::from_markdown`] function for parsing markdown documents
 //!
 //! ## Key Types
 //!
@@ -17,7 +16,7 @@
 //! ### Basic Parsing
 //!
 //! ```
-//! use quillmark_core::decompose;
+//! use quillmark_core::ParsedDocument;
 //!
 //! let markdown = r#"---
 //! title: My Document
@@ -29,49 +28,15 @@
 //! Document content here.
 //! "#;
 //!
-//! let doc = decompose(markdown).unwrap();
+//! let doc = ParsedDocument::from_markdown(markdown).unwrap();
 //! let title = doc.get_field("title")
 //!     .and_then(|v| v.as_str())
 //!     .unwrap_or("Untitled");
 //! ```
 //!
-//! ### Extended Metadata with Tags
-//!
-//! ```
-//! use quillmark_core::decompose;
-//!
-//! let markdown = r#"---
-//! catalog_title: Product Catalog
-//! ---
-//!
-//! # Products
-//!
-//! ---
-//! SCOPE: products
-//! name: Widget
-//! price: 19.99
-//! ---
-//!
-//! A versatile widget for all occasions.
-//! "#;
-//!
-//! let doc = decompose(markdown).unwrap();
-//!
-//! // Access tagged collections
-//! if let Some(products) = doc.get_field("products")
-//!     .and_then(|v| v.as_sequence())
-//! {
-//!     for product in products {
-//!         let name = product.get("name").and_then(|v| v.as_str()).unwrap();
-//!         let price = product.get("price").and_then(|v| v.as_f64()).unwrap();
-//!         println!("{}: ${}", name, price);
-//!     }
-//! }
-//! ```
-//!
 //! ## Error Handling
 //!
-//! The [`decompose`] function returns errors for:
+//! The [`ParsedDocument::from_markdown`] function returns errors for:
 //! - Malformed YAML syntax
 //! - Unclosed frontmatter blocks
 //! - Multiple global frontmatter blocks
@@ -88,6 +53,17 @@ use crate::value::QuillValue;
 /// The field name used to store the document body
 pub const BODY_FIELD: &str = "body";
 
+/// Helper function to convert serde_yaml::Error with location extraction
+fn yaml_error_to_string(e: serde_yaml::Error, context: &str) -> String {
+    let mut msg = format!("{}: {}", context, e);
+
+    if let Some(loc) = e.location() {
+        msg.push_str(&format!(" at line {}, column {}", loc.line(), loc.column()));
+    }
+
+    msg
+}
+
 /// Reserved tag name for quill specification
 pub const QUILL_TAG: &str = "quill";
 
@@ -385,9 +361,7 @@ fn find_metadata_blocks(
 }
 
 /// Decompose markdown into frontmatter fields and body
-pub fn decompose(
-    markdown: &str,
-) -> Result<ParsedDocument, Box<dyn std::error::Error + Send + Sync>> {
+fn decompose(markdown: &str) -> Result<ParsedDocument, Box<dyn std::error::Error + Send + Sync>> {
     // Check input size limit
     if markdown.len() > crate::error::MAX_INPUT_SIZE {
         return Err(format!(
@@ -450,7 +424,7 @@ pub fn decompose(
             HashMap::new()
         } else {
             serde_yaml::from_str(&block.yaml_content)
-                .map_err(|e| format!("Invalid YAML frontmatter: {}", e))?
+                .map_err(|e| yaml_error_to_string(e, "Invalid YAML frontmatter"))?
         };
 
         // Check that all tagged blocks don't conflict with global fields
@@ -483,7 +457,7 @@ pub fn decompose(
             if !block.yaml_content.is_empty() {
                 let yaml_fields: HashMap<String, serde_yaml::Value> =
                     serde_yaml::from_str(&block.yaml_content)
-                        .map_err(|e| format!("Invalid YAML in quill block: {}", e))?;
+                        .map_err(|e| yaml_error_to_string(e, "Invalid YAML in quill block"))?;
 
                 // Check for conflicts with existing fields
                 for key in yaml_fields.keys() {
@@ -520,14 +494,16 @@ pub fn decompose(
             }
 
             // Parse YAML metadata
-            let mut item_fields: HashMap<String, serde_yaml::Value> =
-                if block.yaml_content.is_empty() {
-                    HashMap::new()
-                } else {
-                    serde_yaml::from_str(&block.yaml_content).map_err(|e| {
-                        format!("Invalid YAML in tagged block '{}': {}", tag_name, e)
-                    })?
-                };
+            let mut item_fields: HashMap<String, serde_yaml::Value> = if block
+                .yaml_content
+                .is_empty()
+            {
+                HashMap::new()
+            } else {
+                serde_yaml::from_str(&block.yaml_content).map_err(|e| {
+                    yaml_error_to_string(e, &format!("Invalid YAML in tagged block '{}'", tag_name))
+                })?
+            };
 
             // Extract body for this tagged block
             let body_start = block.end;
