@@ -88,8 +88,8 @@
 //! The workflow supports rendering at three levels:
 //!
 //! 1. **Full render** ([`Workflow::render()`]) - Compose with template → Compile to artifacts (parsing done separately)
-//! 2. **Content render** ([`Workflow::render_source()`]) - Skip parsing, render pre-composed content
-//! 3. **Glue only** ([`Workflow::process_glue_parsed()`]) - Compose from parsed document, return template output
+//! 2. **Content render** ([`Workflow::render_processed()`]) - Skip parsing, render pre-composed content
+//! 3. **Glue only** ([`Workflow::process_glue()`]) - Compose from parsed document, return template output
 //!
 //! ## Examples
 //!
@@ -163,8 +163,8 @@
 //! ```
 
 use quillmark_core::{
-    decompose, Backend, Glue, OutputFormat, ParsedDocument, Quill, RenderError, RenderOptions,
-    RenderResult,
+    Backend, Diagnostic, Glue, OutputFormat, ParsedDocument, Quill, RenderError, RenderOptions,
+    RenderResult, Severity,
 };
 use std::collections::HashMap;
 use std::sync::Arc;
@@ -243,7 +243,8 @@ impl Quillmark {
     /// # impl Backend for CustomBackend {
     /// #     fn id(&self) -> &'static str { "custom" }
     /// #     fn supported_formats(&self) -> &'static [quillmark_core::OutputFormat] { &[] }
-    /// #     fn glue_type(&self) -> &'static str { ".custom" }
+    /// #     fn glue_extension_types(&self) -> &'static [&'static str] { &[".custom"] }
+    /// #     fn allow_auto_glue(&self) -> bool { true }
     /// #     fn register_filters(&self, _: &mut quillmark_core::Glue) {}
     /// #     fn compile(&self, _: &str, _: &quillmark_core::Quill, _: &quillmark_core::RenderOptions) -> Result<quillmark_core::RenderResult, quillmark_core::RenderError> {
     /// #         Ok(quillmark_core::RenderResult::new(vec![], quillmark_core::OutputFormat::Txt))
@@ -260,17 +261,106 @@ impl Quillmark {
     }
 
     /// Register a quill template with the engine by name.
-    pub fn register_quill(&mut self, quill: Quill) {
+    ///
+    /// Validates the quill configuration against the registered backend, including:
+    /// - Backend exists and is registered
+    /// - Glue file extension matches backend requirements
+    /// - Auto-glue is allowed if no glue file is specified
+    /// - Quill name is unique
+    pub fn register_quill(&mut self, quill: Quill) -> Result<(), RenderError> {
         let name = quill.name.clone();
+
+        // Check name uniqueness
+        if self.quills.contains_key(&name) {
+            return Err(RenderError::QuillConfig {
+                diag: Diagnostic::new(
+                    Severity::Error,
+                    format!("Quill '{}' is already registered", name),
+                )
+                .with_code("quill::name_collision".to_string())
+                .with_hint("Each quill must have a unique name".to_string()),
+            });
+        }
+
+        // Get backend
+        let backend_id = quill.backend.as_str();
+        let backend = self
+            .backends
+            .get(backend_id)
+            .ok_or_else(|| RenderError::QuillConfig {
+                diag: Diagnostic::new(
+                    Severity::Error,
+                    format!(
+                        "Backend '{}' specified in quill '{}' is not registered",
+                        backend_id, name
+                    ),
+                )
+                .with_code("quill::backend_not_found".to_string())
+                .with_hint(format!(
+                    "Available backends: {}",
+                    self.backends.keys().cloned().collect::<Vec<_>>().join(", ")
+                )),
+            })?;
+
+        // Validate glue_file extension or auto_glue
+        if let Some(glue_file) = &quill.metadata.get("glue_file").and_then(|v| v.as_str()) {
+            let extension = std::path::Path::new(glue_file)
+                .extension()
+                .and_then(|e| e.to_str())
+                .map(|e| format!(".{}", e))
+                .unwrap_or_default();
+
+            if !backend.glue_extension_types().contains(&extension.as_str()) {
+                return Err(RenderError::QuillConfig {
+                    diag: Diagnostic::new(
+                        Severity::Error,
+                        format!(
+                            "Glue file '{}' has extension '{}' which is not supported by backend '{}'",
+                            glue_file, extension, backend_id
+                        ),
+                    )
+                    .with_code("quill::glue_extension_mismatch".to_string())
+                    .with_hint(format!(
+                        "Supported extensions for '{}' backend: {}",
+                        backend_id,
+                        backend.glue_extension_types().join(", ")
+                    )),
+                });
+            }
+        } else {
+            if !backend.allow_auto_glue() {
+                return Err(RenderError::QuillConfig {
+                    diag: Diagnostic::new(
+                        Severity::Error,
+                        format!(
+                            "Backend '{}' does not support automatic glue generation, but quill '{}' does not specify a glue file",
+                            backend_id, name
+                        ),
+                    )
+                    .with_code("quill::auto_glue_not_allowed".to_string())
+                    .with_hint(format!(
+                        "Add a glue file with one of these extensions: {}",
+                        backend.glue_extension_types().join(", ")
+                    )),
+                });
+            }
+        }
+
         self.quills.insert(name, quill);
+        Ok(())
     }
 
     /// Load a workflow from a parsed document that contains a quill tag
     pub fn workflow_from_parsed(&self, parsed: &ParsedDocument) -> Result<Workflow, RenderError> {
         let quill_name = parsed.quill_tag().ok_or_else(|| {
-            RenderError::Other(
-                "No QUILL field found in parsed document. Add `QUILL: <name>` to the markdown frontmatter.".into(),
-            )
+            RenderError::UnsupportedBackend {
+                diag: Diagnostic::new(
+                    Severity::Error,
+                    "No QUILL field found in parsed document. Add `QUILL: <name>` to the markdown frontmatter.".to_string(),
+                )
+                .with_code("engine::missing_quill_tag".to_string())
+                .with_hint("Add QUILL: <name> to specify which quill template to use".to_string()),
+            }
         })?;
         self.workflow_from_quill_name(quill_name)
     }
@@ -286,9 +376,19 @@ impl Quillmark {
         let quill = match quill_ref {
             QuillRef::Name(name) => {
                 // Look up the quill by name
-                self.quills.get(name).ok_or_else(|| {
-                    RenderError::Other(format!("Quill '{}' not registered", name).into())
-                })?
+                self.quills
+                    .get(name)
+                    .ok_or_else(|| RenderError::UnsupportedBackend {
+                        diag: Diagnostic::new(
+                            Severity::Error,
+                            format!("Quill '{}' not registered", name),
+                        )
+                        .with_code("engine::quill_not_found".to_string())
+                        .with_hint(format!(
+                            "Available quills: {}",
+                            self.quills.keys().cloned().collect::<Vec<_>>().join(", ")
+                        )),
+                    })?
             }
             QuillRef::Object(quill) => {
                 // Use the provided quill directly
@@ -301,18 +401,32 @@ impl Quillmark {
             .metadata
             .get("backend")
             .and_then(|v| v.as_str())
-            .ok_or_else(|| {
-                RenderError::Other(
-                    format!("Quill '{}' does not specify a backend", quill.name).into(),
+            .ok_or_else(|| RenderError::EngineCreation {
+                diag: Diagnostic::new(
+                    Severity::Error,
+                    format!("Quill '{}' does not specify a backend", quill.name),
                 )
+                .with_code("engine::missing_backend".to_string())
+                .with_hint(
+                    "Add 'backend = \"typst\"' to the [Quill] section of Quill.toml".to_string(),
+                ),
             })?;
 
         // Get the backend by ID
-        let backend = self.backends.get(backend_id).ok_or_else(|| {
-            RenderError::Other(
-                format!("Backend '{}' not registered or not enabled", backend_id).into(),
-            )
-        })?;
+        let backend =
+            self.backends
+                .get(backend_id)
+                .ok_or_else(|| RenderError::UnsupportedBackend {
+                    diag: Diagnostic::new(
+                        Severity::Error,
+                        format!("Backend '{}' not registered or not enabled", backend_id),
+                    )
+                    .with_code("engine::backend_not_found".to_string())
+                    .with_hint(format!(
+                        "Available backends: {}",
+                        self.backends.keys().cloned().collect::<Vec<_>>().join(", ")
+                    )),
+                })?;
 
         // Clone the Arc reference to the backend and the quill for the workflow
         let backend_clone = Arc::clone(backend);
@@ -369,28 +483,28 @@ impl Workflow {
         parsed: &ParsedDocument,
         format: Option<OutputFormat>,
     ) -> Result<RenderResult, RenderError> {
-        let glue_output = self.process_glue_parsed(parsed)?;
+        let glue_output = self.process_glue(parsed)?;
 
         // Prepare quill with dynamic assets
         let prepared_quill = self.prepare_quill_with_assets();
 
         // Pass prepared quill to backend
-        self.render_source_with_quill(&glue_output, format, &prepared_quill)
+        self.render_processed_with_quill(&glue_output, format, &prepared_quill)
     }
 
     /// Render pre-processed glue content, skipping parsing and template composition.
-    pub fn render_source(
+    pub fn render_processed(
         &self,
         content: &str,
         format: Option<OutputFormat>,
     ) -> Result<RenderResult, RenderError> {
         // Prepare quill with dynamic assets
         let prepared_quill = self.prepare_quill_with_assets();
-        self.render_source_with_quill(content, format, &prepared_quill)
+        self.render_processed_with_quill(content, format, &prepared_quill)
     }
 
     /// Internal method to render content with a specific quill
-    fn render_source_with_quill(
+    fn render_processed_with_quill(
         &self,
         content: &str,
         format: Option<OutputFormat>,
@@ -415,33 +529,64 @@ impl Workflow {
         self.backend.compile(content, quill, &render_opts)
     }
 
-    /// Process Markdown through the glue template without compilation, returning the composed output.
-    pub fn process_glue(&self, markdown: &str) -> Result<String, RenderError> {
-        let parsed_doc = decompose(markdown).map_err(|e| RenderError::InvalidFrontmatter {
-            diag: quillmark_core::error::Diagnostic::new(
-                quillmark_core::error::Severity::Error,
-                format!("Failed to parse markdown: {}", e),
-            ),
-            source: Some(anyhow::anyhow!(e)),
-        })?;
+    /// Process a parsed document through the glue template without compilation
+    pub fn process_glue(&self, parsed: &ParsedDocument) -> Result<String, RenderError> {
+        // Validate document against schema
+        self.validate_document(parsed)?;
 
-        self.process_glue_parsed(&parsed_doc)
+        // Create appropriate glue based on whether template is provided
+        let mut glue = match &self.quill.glue {
+            Some(s) if !s.is_empty() => Glue::new(s.to_string()),
+            _ => Glue::new_auto(),
+        };
+        self.backend.register_filters(&mut glue);
+        let glue_output =
+            glue.compose(parsed.fields().clone())
+                .map_err(|e| RenderError::TemplateFailed {
+                    diag: Diagnostic::new(Severity::Error, e.to_string())
+                        .with_code("template::compose".to_string()),
+                })?;
+        Ok(glue_output)
     }
 
-    /// Process a parsed document through the glue template without compilation
-    pub fn process_glue_parsed(&self, parsed: &ParsedDocument) -> Result<String, RenderError> {
-        // Create appropriate glue based on whether template is provided
-        let mut glue = if self.quill.glue_template.is_empty() {
-            Glue::new_json()
-        } else {
-            Glue::new(self.quill.glue_template.clone())
+    /// Validate a ParsedDocument against the Quill's schema
+    ///
+    /// Validates the document's fields against the schema defined in the Quill.
+    /// The schema can come from either:
+    /// - A json_schema_file specified in Quill.toml
+    /// - TOML `[fields]` section converted to JSON Schema
+    ///
+    /// If no schema is defined, this returns Ok(()).
+    pub fn validate(&self, parsed: &ParsedDocument) -> Result<(), RenderError> {
+        self.validate_document(parsed)
+    }
+
+    /// Internal validation method
+    fn validate_document(&self, parsed: &ParsedDocument) -> Result<(), RenderError> {
+        use quillmark_core::validation;
+
+        // Build or load JSON Schema
+
+        if self.quill.schema.is_null() {
+            // No schema defined, skip validation
+            return Ok(());
         };
 
-        self.backend.register_filters(&mut glue);
-        let glue_output = glue
-            .compose(parsed.fields().clone())
-            .map_err(RenderError::from)?;
-        Ok(glue_output)
+        // Validate document
+        match validation::validate_document(&self.quill.schema, parsed.fields()) {
+            Ok(_) => Ok(()),
+            Err(errors) => {
+                let error_message = errors.join("\n");
+                Err(RenderError::ValidationFailed {
+                    diag: Diagnostic::new(Severity::Error, error_message)
+                        .with_code("validation::document_invalid".to_string())
+                        .with_hint(
+                            "Ensure all required fields are present and have correct types"
+                                .to_string(),
+                        ),
+                })
+            }
+        }
     }
 
     /// Get the backend identifier (e.g., "typst").
@@ -478,11 +623,15 @@ impl Workflow {
         // Check for collision
         if self.dynamic_assets.contains_key(&filename) {
             return Err(RenderError::DynamicAssetCollision {
-                filename: filename.clone(),
-                message: format!(
-                    "Dynamic asset '{}' already exists. Each asset filename must be unique.",
-                    filename
-                ),
+                diag: Diagnostic::new(
+                    Severity::Error,
+                    format!(
+                        "Dynamic asset '{}' already exists. Each asset filename must be unique.",
+                        filename
+                    ),
+                )
+                .with_code("workflow::asset_collision".to_string())
+                .with_hint("Use unique filenames for each dynamic asset".to_string()),
             });
         }
 
@@ -525,11 +674,15 @@ impl Workflow {
         // Check for collision
         if self.dynamic_fonts.contains_key(&filename) {
             return Err(RenderError::DynamicFontCollision {
-                filename: filename.clone(),
-                message: format!(
-                    "Dynamic font '{}' already exists. Each font filename must be unique.",
-                    filename
-                ),
+                diag: Diagnostic::new(
+                    Severity::Error,
+                    format!(
+                        "Dynamic font '{}' already exists. Each font filename must be unique.",
+                        filename
+                    ),
+                )
+                .with_code("workflow::font_collision".to_string())
+                .with_hint("Use unique filenames for each dynamic font".to_string()),
             });
         }
 
